/****************************************************************
**
** Attal : Lords of Doom
**
** tavern.cpp
** Manage tavern building
**
** Version : $Id: tavern.cpp,v 1.3 2005/02/12 22:00:02 audoux Exp $
**
** Author(s) : Pascal Audoux
**
** Date : 04/02/2005
**
** Licence :    
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/

#include "tavern.h"
 
// generic include files
// include files for QT
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include <qsignalmapper.h>
// application specific include files
#include "libCommon/attalSocket.h"
#include "libCommon/dataTheme.h"
#include "libCommon/genericLord.h"
#include "libCommon/log.h"

#include "libClient/gui.h"
#include "libClient/imageTheme.h"
#include "libClient/widget.h"

extern DataTheme DataTheme;
extern ImageTheme ImageTheme;

//
// ----- Tavern -----
//

Tavern::Tavern( QWidget * parent )
: QDialog( parent, 0, true )
{
	_base = 0;
	_lords.setAutoDelete( true );
	_signalMapper = 0;
	
	QLabel * labIntro = new QLabel( this );
	labIntro->setAlignment( Qt::WordBreak );
	labIntro->setText( "Some lords are visiting your base and are available in your tavern. Here is the list:" );
	FIXEDSIZE( labIntro );
	
	
	QPushButton * pbOk = new QPushButton( this );
	pbOk->setText( tr( "Ok" ) );
	FIXEDSIZE( pbOk );
	
	QHBoxLayout * layHOk = new QHBoxLayout();
	layHOk->addStretch( 1 );
	layHOk->addWidget( pbOk );
	layHOk->addStretch( 1 );
	
	_scroll = new QScrollView( this );
	
	QVBoxLayout * layout = new QVBoxLayout( this );
	layout->setSpacing( 5 );
	layout->setMargin( 5 );
	layout->addWidget( labIntro );
	layout->addWidget( _scroll, 1 );
	layout->addLayout( layHOk );
	layout->activate();
	
	connect( pbOk, SIGNAL( clicked() ), SLOT( accept() ) );
}

void Tavern::initSocket( AttalSocket * socket )
{
	_socket = socket;
}

void Tavern::handleSocket()
{
	if( ( _socket->getCla1() == SO_GAME ) and ( _socket->getCla2() == C_GAME_TAVERN ) ) {
		switch( _socket->getCla3() ) {
		case C_TAVERN_INFO:
			handleTavernInfo();
			break;
		case C_TAVERN_LORD:
			handleTavernLord();
			break;
		default:
			logEE( "Should not happen" );
		}	
	}
}

void Tavern::handleTavernInfo()
{
	_nbLord = _socket->readChar();
	_nbLordRead = 0;
	_lords.clear();
	if( _signalMapper ) {
		delete _signalMapper;
	}
	_signalMapper = new QSignalMapper( this );
	connect( _signalMapper, SIGNAL( mapped( int ) ), SLOT( slot_buy( int ) ) );
	if( _nbLordRead >= _nbLord ) {
		QDialog::exec();
	} else {
		_socket->askTavernLord( _base, _nbLordRead );
	}
}

void Tavern::handleTavernLord()
{
	_nbLordRead++;
	/// XXX: reads lord info and displays it
	uint idLord = _socket->readInt();
	GenericLord * lord = new GenericLord();
	lord->setId( idLord );
	TavernLord * widget = new TavernLord( _scroll->viewport() );
	widget->init( lord );
	_signalMapper->setMapping( widget, _lords.count() );
	_lords.append( widget );
	_scroll->addChild( widget );
	connect( widget, SIGNAL( sig_buy() ), _signalMapper, SLOT( map() ) );
	if( _nbLordRead >= _nbLord ) {
		QDialog::exec();
	}
}

void Tavern::start( GenericBase * base )
{
	_base = base;
	/// XXX: test if we need ask server info...
	_socket->askTavernInfo( base );
}

void Tavern::slot_buy( int numLord )
{
	/// XXX: not finished
	logDD( "buy %d", numLord );
}

//
// ----- TavernLord -----
//

TavernLord::TavernLord( QWidget * parent )
: QWidget( parent )
{
	_photo = new Icon( this );
	
	_info = new QLabel( this );
	_price = new QLabel( this );
	
	QVBoxLayout * layV1 = new QVBoxLayout();
	layV1->setMargin( 5 );
	layV1->setSpacing( 5 );
	layV1->addWidget( _info );
	layV1->addWidget( _price );
	layV1->addStretch( 1 );
	
	QPushButton * pbBuy = new QPushButton( this );
	pbBuy->setText( tr( "Buy" ) );
	FIXEDSIZE( pbBuy );
	
	QHBoxLayout * layout = new QHBoxLayout( this );
	layout->setSpacing( 5 );
	layout->setMargin( 5 );
	layout->addWidget( _photo );
	layout->addLayout( layV1, 1 );
	layout->addWidget( pbBuy );
	layout->activate();
	
	connect( pbBuy, SIGNAL( clicked() ), this, SIGNAL( sig_buy() ) );
}

void TavernLord::init( GenericLord * lord )
{
	QString temp;
	_lord = lord;
	
	if( ImageTheme.getLordPixmap( lord->getId() ) ) {
		_photo->setPixmap( * ImageTheme.getLordPixmap( lord->getId() ) );
	}
	
	temp.sprintf( "Lord %s (%s)", lord->getName().latin1(), lord->getCategoryName().latin1() );
	_info->setText( temp );
	FIXEDSIZE( _info );
	
	/// XXX: to change
	_price->setText( "2500 gold" );
	FIXEDSIZE( _price );
}



