/****************************************************************
**
** Attal : Lords of Doom
**
** scenarioEditor.cpp
** main file for the scenario editor
**
** Version : $Id: scenarioEditor.cpp,v 1.9 2005/03/10 14:16:21 lusum Exp $
**
** Author(s) : Pascal Audoux
**
** Date : 27/08/2000
**
** Licence :    
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/



#include <iostream>
using namespace std;
// include files for QT
#include <qapplication.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>

// application specific includes
#include "scenarioEditor/scenarioInterface.h"
#include "libCommon/log.h"
#include "libClient/map.h"
#include "libClient/gui.h"
#include "conf.h"

Map * theMap;

QString IMAGE_PATH( IMAGE_DEFAULT_PATH );

QString MUSIC_PATH( MUSIC_DEFAULT_PATH );

QString SOUND_PATH( SOUND_DEFAULT_PATH );

QString DATA_PATH( DATA_DEFAULT_PATH );

QString CAMPAIGN_PATH( CAMPAIGN_DEFAULT_PATH );

QString SCENARIO_PATH( SCENARIO_DEFAULT_PATH );

QString THEME( THEME_DEFAULT );

#ifdef WITH_SOUND
extern bool ATT_SOUND;
#endif

int main( int argc, char **argv )
{
	QApplication app( argc, argv );
	QTranslator translator( 0 );
	translator.load( QString("scenario_editor_") + QTextCodec::locale(), "." );

	app.installTranslator( &translator );

	for( int i = 1; i < argc; i++ ) {
		if ( strcmp( argv[i], "--theme" ) == 0 ) {
			i++;
			if( i < argc ) {
				THEME = QString( argv[i] );
			}
		} else if( ( strcmp( argv[i], "-v" ) == 0 )  ) {	
			int level;
			level = atoi(argv[i+1]) ;
			setLogLevel((LogLevel) level ) ;
		} else if( ( strcmp( argv[i], "--nosound")  == 0 ) ) {
			#ifdef WITH_SOUND
			ATT_SOUND = false;
			#endif
		} else if( ( strcmp( argv[i], "-help" ) == 0 ) || ( strcmp( argv[i], "--help" ) == 0 ) ) {
			cout << "Usage: scenarioEditor [OPTION]"  << endl;
			cout << "Start scenarioEditor "  << endl;
			cout << endl;
			cout << "  --theme [NAME]      Load a particular theme (Medieval by default)"  << endl;
			cout << "  -v [LEVEL]                  Level of error messages (0-4) (default 4)"  << endl;
			#ifdef WITH_SOUND
			cout << "  --nosound           Disable totally sound (ex: for hardware problems)"  << endl;
			#endif
			cout << "  --help              This screen"  << endl;
			return 1; 
		}
	}	

	useTheme();
	theMap = 0;

	ScenarioInterface edit;
	app.setMainWidget( &edit );
	edit.show();

	return app.exec();	
}
