/****************************************************************
**
** Attal : Lords of Doom
**
** screen.h
** main widget of the editor
**
** Version : $Id: screen.h,v 1.13 2004/07/31 19:29:47 audoux Exp $
**
** Author(s) : Pascal Audoux
**
** Date : 27/08/2000
**
** Licence :    
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/


#ifndef SCREEN_H
#define SCREEN_H


// generic include files
// include files for QT
#include <qstring.h>
#include <qwidget.h>
// application specific include files
#include "scenarioEditor/graphicalGameData.h"

class QPopupMenu;

class Cell;
class GenericCell;
class ItemSelector;
class MapView;
class MiniMap;
 
/*              ------------------------------
 *                         Screen
 *              ------------------------------ */

enum POPUP_ITEM {
	M_DELETE,
	M_PROPERTIES
};

/** comment for the class */
class Screen : public QWidget, public GraphicalGameData
{
	Q_OBJECT
public:
	/** Constructor */
	Screen( QWidget * parent = 0, char * name = 0 );

	~Screen();
	
	/** Display Info Scenario dialog */
	void displayInfo();
	
	/** Load scenario */
	bool load( const QString filename );
	
	/** Save scenario */
	void save( const QString filename );

	/** Create new scenario */
	void newScenario();
	
	/** clear the current map */
	void clearMap();
	
	/** fill the current map with the tile 'tile' and diversification 'divers' (0 means random) */
	void fillMap( uint tile, uint divers );

	void displayPlayersProperties();

	void handleDelete();

	void manageQuests();

	void displayMiniMap( bool state );

public slots:
	/** Slot when mouse button pressed */
	void slot_mouseLeftPressed( Cell * c );
	void slot_mouseRightPressed( Cell * c );

	/** Slot when mouse button is released */
	void slot_mouseReleased();

	/** Handle popup menu */
	void slot_popup( int );
	
	/** Slot when mouse moved */
	void slot_mouseMoved( Cell * c );

	/** Slot for center map on a cell */
	void slot_centerMinimap( GenericCell * c );

signals:
	void sig_status( const QString & );

private:
	
	void handleProperties();
	void handlePropertiesLord();
	void handlePropertiesBase();
	void handlePropertiesBuilding();
	void handlePropertiesCreature();
	void handlePropertiesEvent();
	void updateStatusBar( Cell * c );
	
	bool _leftPressed;
	ItemSelector * _selector;
	Cell * _currentCell;
	MapView * _view;
	MiniMap * _miniMap;
	
	QPopupMenu * _menu;
};
 
#endif // SCREEN_H   
